# pio_off_delay.py - 12.09.2021

# Eine Ausschaltverzögerung (Treppenautomat) wird durch einen
# Tastendrock ausgelöst.
# Der Taster zieht den Input-Pin auf high !
# Weiterer Tastendruck während der Verzögerungszeit startet den
# Zeitzähler neu.
# Die Verzögerungszeit ist jederzeit veränderbar, wird aber erst
# bei dem nächsten Schaltvorgang oder bei einem Retrigger benutzt.
# Die verbleibende Restzeit kann ausgelsen werden.

import time
from rp2 import PIO, StateMachine, asm_pio
from machine import Pin

@asm_pio(set_init=PIO.OUT_LOW, in_shiftdir=rp2.PIO.SHIFT_LEFT,)

def off_delay():
    #set(y, 5)
    #mov(osr, y)              #    Die Wartezeit in 0,5 Sekunden steht in OSR
   
    wrap_target()
                              
    wait(1, pin, 0)           #   1 Warten auf High an Taster
    label("retrigger")
   
    set(x, 5)                 #   2 Auszeit, in der nicht auf die Taste reagiert wird
                              #     zum Entprellen und gegen Dauerdruck auf Taste
    label("wait")
    jmp(pin, "wait")     [10] #   3
    jmp(x_dec, "wait")    [5] #   4 20 * 20 * 0.5 ms = 200 ms  
   
    set(pins, 1)              #   5

    mov(y, osr)               #   6
    label("outer") 
    set(x, 31)            [6] #   7 Warteschleife 500ms
    
    label("inner")
    jmp(pin, "retrigger")     #   8
    jmp(x_dec, "inner")  [30] #   9 (6 + 1) + (31 * (1 + 1 + 30))
    
    jmp(y_dec, "outer")       #  10
    
    set(pins, 0)              #  11
    wrap()
    

class Off_Delay(object):
    def __init__(self, statemachine, input_pin, output_pin, default_time=5):
        self.sm = StateMachine(statemachine, off_delay,
                            set_base=Pin(output_pin), 
                            in_base=Pin(input_pin, Pin.IN, Pin.PULL_DOWN),
                            jmp_pin=Pin(input_pin, Pin.IN, Pin.PULL_DOWN),
                            freq=2000)
        self.sm.restart()
        self.sm.active(1)
        self.sm.put(default_time * 2 - 1)
        self.sm.exec("pull()")

    def set_delay(self, delay):
        # Setzt die Verzögerungszeit in der Einheit "Sekunden"
        self.sm.put(delay * 2 - 1)
        self.sm.exec("pull()")
        
    def get_delay(self):
        # Gibt die verbleibende Verzügerungszeit zurück
        # die steht im Y-Register
        # wenn die Zeit abgelaufen, dann steht dort 0xFFFF_FFFF
        self.sm.exec("mov(isr, y)")
        self.sm.exec("push()")
        if self.sm.rx_fifo():
            x = self.sm.get()
            #print("{:08x}".format(x))
            # Wenn das höhstwertige Bit gesetzt
            if x & 0x8000_0000:
                print(0)
            else:
                print((x + 1) // 2)


if __name__ == "__main__":
    
    input_pin = 17
    output_pin = 0
    
    t = Off_Delay(statemachine=0, input_pin=input_pin, output_pin=output_pin)
    
    print("Press key to start delay ...\n")
    
    while True:
        t.get_delay()
        time.sleep(1)
        
    print("\ndone")
