# pio_onoff_delay.py - 12.09.2021
#
# Schaltet über einen Taster einen Ausgang mit optionaler Ein- und
# Ausschalt-Verzögerung.
# Eine aktive Verzögerugszeit wird an einem zusätzlichen "Pin" (.side(2)(
# angezeigt.
# Der Taster zieht den Input-Pin auf high !
# Durch sideset wird die notwendige Anzahl der INstruktionen reduziert,

from machine import Pin
from rp2 import PIO, StateMachine, asm_pio
   
class On_Off_Delay(object):
    def __init__(self, statemachine, input_pin, sideset_pin):
        self.sm = StateMachine(statemachine, self.on_off_delay,
                    in_base=Pin(input_pin, Pin.IN, Pin.PULL_DOWN),
                    jmp_pin=Pin(input_pin, Pin.IN, Pin.PULL_DOWN),
                    sideset_base=Pin(sideset_pin),
                    freq=2000)
        
    def set_delay(self, on_delay=0, off_delay=0):
        # Schreibt die Zeiten für die Verzögerung in die Register
        # ISR (Einschaltverzögerung) und OSR (Ausschaltverzögerung).
        # Angabe in 1/10 Sekunden: Der Counter teilt durch 4,
        # Bei F_PIO 2000 Hz entsprechen 500 Takte -> 1 Sekunde.
        self.sm.put(on_delay * 50)   
        self.sm.exec("pull()")
        self.sm.exec("mov(isr, osr)")
        self.sm.put(off_delay * 50)
        self.sm.exec("pull()")
        
    def activate(self, state=1):
        self.sm.active(1 if state else 0)

    @staticmethod
    @asm_pio(sideset_init=(PIO.OUT_LOW,) * 2)
    def on_off_delay():
                                    #     im Register  ISR muss die Einschaltverzögerung
                                    #     im Regiester OSR die Ausschaltverzögerung stehen
                                    #     X-Register für Entprellen
                                    #     Y-Register für Wartezeiten
                                    #     Das maximales Delay bei 2 Sidesets ist [3]    
        wrap_target()                                    
        mov(y, isr)  .side(0)       #   1 Einschaltverzögerung laden,
                                    #     und Ausschalten       
                                    #     Ausgangspunkt: der Pin ist low
        wait(1, pin, 0)             #   2 Warten auf einen Tastendruck / Pin high
        
        label("wait_on_a")          #     Warten bis die Taste wieder losgelassen ist
        set(x, 15) .side(2)         #   3 und/oder nicht mehr prellt
                                    #     .side schaltet Kontrollled
        label("wait_on")
        jmp(pin, "wait_on_a")   [3] #   4

        jmp(x_dec, "wait_on")   [3] #   5 
            
        label("on_next")
        jmp(y_dec, "on_next")   [3] #   6
                                    #     Die Wartezeit ist abgelaufen 
        mov(y, osr) .side(1)        #   7 Einschaltverzögerung laden
                                    #     und Einschalten    
        wait(1, pin, 0)             #   8 Warten auf Tastendruck / Pin high
      
        label("wait_off_a")         #     Warten bis die Taste wieder losgelassen ist
        set(x, 14)  .side(3)        #   9 und/oder nicht mehr prellt
                                    #     .side schaltet Kontrollled 
        label("wait_off")
        jmp(pin, "wait_off_a")  [3] #  10 
        jmp(x_dec, "wait_off")  [3] #  11 
             
        label("off_next")           #     Ausschaltverzögerung
        jmp(y_dec, "off_next")  [3] #  12
        
        wrap()

if __name__ == "__main__":
    
    on_delay  = 10        # Delay in 1/109 Sekunden
    off_delay = 10
    
    on_off = On_Off_Delay(statemachine=7, input_pin=17, sideset_pin=0)
    on_off.set_delay(on_delay=on_delay, off_delay=off_delay)
    on_off.activate(1)
    
    print("Press key to start delay")
    print("running on statemachine ...")