# pio_pull_thresh.py - 12.09.2021
#
# Testen, welchen Effekt pull_thresh ihne autopull auf out()
# in Verbindung mit jmp(not_osre) hat.
# Ergebnis: Es werden zwar die vollständigen 32 Bit ins OSR
# übertragen und bei dem nachfolgenden Code
# - pull()
# - mov(isr, osr)
# - push()
# wieder mit vollen 32 Bit zurückgegeben.
# Pull_thresh beeinflusst also nicht die Datenbreite im OSR
#
# Aber out() begrenzt die Ausgabe tatsächlich auf die Anzahl
# der mit pull_thresh definierten Bits.
#
# Der Test:
# Es wird eine Zahl an die Statemachine übergeben.
# Die Bits werden nacheinander ins X-Register geschrieben
# und sofort wieder im ISR-Register eingelesen - solange,
# bis not_osre nicht mehr erfüllt ist.
# Der Wert in ISR muss dann identisch sein mit den
# durch pull_thresh festgelegten rechtseitigen Bits des OSR.
# Allerdings mtehen werden die Bits von links nach rchts in ISR
# geshiftet, stehen also linksbündig:
# input:  0x0000_00AA (mit pull_thresh = 8) führt zu
# output: 0xAA00_0000 
#
# Pull_thresh setzt offensichtlich nur den internen Bit-Zähler,
# der bei jedem out() decremeentiert und bei jmp(not_osre."label")
# geprüft wird.
# Der Status von "autopull" hat auf dieses Verhalten keine Auswirkung.


import time
from rp2 import PIO, StateMachine, asm_pio
from machine import Pin

PULL_THRESH = 8

@asm_pio(autopull=False,
         pull_thresh=PULL_THRESH,
         out_shiftdir=PIO.SHIFT_RIGHT,
         in_shiftdir=rp2.PIO.SHIFT_RIGHT)

def pull_push():
    
    wrap_target()
    pull()       
    label("next")
    out(x, 1)         # Bitweise nach X ausgeben
    in_(x, 1)         # und bittweise ins ISR zurück
    jmp(not_osre, "next")
    push()
    wrap()
    
sm = StateMachine(0, pull_push, freq=2000)
sm.active(1)

out = 0x12345678
sm.put(out)
    
time.sleep(.1)
print("pull_thresh = {}".format(PULL_THRESH))
print("OSR : {:08x}".format(out))
print("ISR : {:08x}".format(sm.get()))

time.sleep(.2)
print("\ndone")
