# pio_wave_pattern_led_blink.py - 12.09.2021
#
# Blinkende Led mit änderbaren Blinkfolge
# mit Hilfe rausgeshifteten Bitmustern.
# Der Fehler, der durch das zusätzliche pull()
# entsteht, ist wegen der delays relativ gering:
# bei insgesamt 32 * 64 Takten = 2048 Takte
# fällt 1 zusätzlicher Takt an.

from rp2 import PIO, StateMachine, asm_pio
from machine import Pin, lightsleep

@asm_pio(autopull=True,
         pull_thresh=32,
         out_init=PIO.OUT_LOW,
         out_shiftdir=PIO.SHIFT_RIGHT)

def wave():  
    wrap_target()

    pull(noblock)              # 1 Wert aus X Register holen
    label("next")
    out(pins, 1)          [31] # 2 Bitweise nach X ausgeben
    jmp(not_osre, "next") [31] # 3 
    
    wrap()
    
# -------------------------------------------------------

# 32 * (32 + 32) + 1 = 2049 Takte pro oull()
# Gewählte F_PIO = 2059 - > 1 Sekunde pro pull

led_pin = 0

# - Blinkmuster - Ablaufdauer jeweils 1 Sekunde
blink_1Hz = 0x0000_FFFF
blink_2Hz = 0x00FF_00FF
blink_4Hz = 0x0F0F_0F0F
blink_8Hz = 0x3333_3333
flash_1   = 0xF
flash_2   = 0xF0F
flash_3   = 0b1111_00001111_00001111

def set_pattern(pattern):
    sm.active(0)
    sm.restart()
    sm.put(pattern)        # pattern ins X-Register kopieren
    sm.exec("pull()")
    sm.exec("mov(x, osr)")
    sm.active(1)
    
sm = StateMachine(0, wave, out_base = Pin(led_pin), freq=2_049)

try:
    while True:
        for pattern in (blink_1Hz, blink_2Hz, blink_4Hz, blink_8Hz, flash_1, flash_2):
            set_pattern(pattern)
            lightsleep(5000)
            
except KeyboardInterrupt:
    set_pattern(0)
    lightsleep(10)     # damit Zeit zum Ausschalten der Led bleibt
    sm.active(0)
    
print("\ndone")
