# pio_wave_pattern_1.py - 12.09.2021
#
# Wave-Generator mit 32-Bit-Patterns, die per out() ausgegeben werden.
# Achtung:
# Wegen des pull() wird nach dem letzten Bit ein zusätzlicher Takt benötigt !
# Vergkeich zur Methode mit sideset:
#
#    Variante mit autopull     Variante mit sideset
#    pull(noblock)           
#    label("next")             label("next")
#    out(pins, 1)              nop()        .side(1)
#    jmp(not_osre, "next")     jmp("next")  .side(0)
#
#    (bei Pattern 0x0000FFFF)
#    F_out = F_PIO / 32        F_put = F_PIO / 2
#
# Die Variante mit autopull ist zwar um 1 Instruktion länger,
# aber:
# - es sind niedrigere Frequenzen ohne Zusatzaufwand erreichbar
#   bei F_PIO von 2049, delay von 31 -> 1Hz

#   2049 / (32 * (32 + 32) + 1) = 2049 / 2048 + 1 ? 1Hz
# - Die Ausgabe ist im Rahmen des Bitmusters jederzeit veränderbar
#   Bei eave_sideset ist nur ein delay bis 7 möglich
#   2048 / 2 * (1 + 7) = 2048 / 16 = 128 Hz

import time
from rp2 import PIO, StateMachine, asm_pio
from machine import Pin

# -------------------------------------------------
# Variante mit autopull + out()
@asm_pio(autopull=True,
         pull_thresh=32,
         out_init=PIO.OUT_LOW,
         out_shiftdir=PIO.SHIFT_RIGHT)

def wave_pull():
    D = 31                     #   delay [0 ... 31]
    wrap_target()

    pull(noblock)             # 1 Wert aus X Register holen
    label("next")
    out(pins, 1)          [D] # 2 Bitweise nach X ausgeben
    jmp(not_osre, "next") [D] # 3 
    
    wrap()
    
# ---------------------------------------
# Variante mit sideset
@asm_pio(sideset_init=PIO.OUT_LOW)
 
def wave_sideset():
    D = 7                        #   delay [0 ... 7]
    label("next")
    nop()          .side(1) [D]  # 1
    jmp("next")    .side(0) [D]  # 2
 
# -------------------------------------------------------

out_pin = 0

#pattern = 0b1110011_00110011_10101010_00101010
pattern = 0x5555_5555   # maximal erreichbare Frequenz
pattern = 0x0000_FFFF   # niedrigst erreichbare Frequenu 
 
sm = StateMachine(1, wave_sideset, sideset_base=Pin(out_pin), freq=2048)
#sm = StateMachine(0, wave_pull, out_base = Pin(out_pin), freq=2049)
sm.put(pattern)        # pattern ins X-Register kopieren
sm.exec("pull()")
sm.exec("mov(x, osr)")

sm.active(1)
time.sleep(1.5)
sm.active(0)
    
print("\ndone")
