# pio_wave_pattern_2.py - 12.09.2021
#
# Wave-Generator aus 32-Bit-Pattern, das per out() ausgegeben wird.
# Um den Fehler auszugleichen, der durch den zusätzlichen Takt beim
# pull() entsteht:
# Es werden nur 31 Bit rausgeshiftet (pull_thresh = 31 !).
# Die beiden "fehlenden" Takte übernimmt pull() [1].
# Das hat zur Folge, dass der 32. Takt immer gleich dem 31.Takt ist ! 

# (Pulse + Pause) je 2 Takte = 8 Takte 
# F_PIO = F_OUT *  8 bei pattern = 0x3333_3333
# F_PIO = F_OUT * 32 bei pattern = 0x00FF_00FF
# F_PIO = F_OUT * 64 bei pattern = 0x0000_FFFF

# delay = [0 .. 31]
# F_out = (F_PIO  / (64 * (1 + delay)) * ( ticks / fifo_word)
# bei delay = 31 und F_PIO = 2048:
# 1Hz   =  (2048   / (64 * 32)) * 1 -> bei 0x0000_FFFF
# 2Hz   =  (2048   / (64 * 32)) * 2 -> bei 0x00FF_00FF
# 4Hz   =  (2048   / (64 * 32)) * 4 -> bei 0x0F0F_0F0F
# 8Hz   =  (2048   / (64 * 32)) * 8 -> bei 0x3333_3333

import time
from rp2 import PIO, StateMachine, asm_pio
from machine import Pin

@asm_pio(autopull=True,
         pull_thresh=31,
         out_init=PIO.OUT_LOW,
         out_shiftdir=PIO.SHIFT_RIGHT)

def wave():
    D = 31                       # delay = [0 ... 31]
    wrap_target()

    pull(noblock)           [D] # 1 Wert aus X Register holen
    nop()                   [D] #   nur notwendg ab D > 15
                                #   weil die doppelte Anzahl von delays
                                #   für pull() notwendig ost.
    label("next")
    out(pins, 1)            [D] # 2 Bitweise nach X ausgeben
    jmp(not_osre, "next")   [D] # 3 
    
    wrap()
    
# -------------------------------------------------------
#            01  -> die Bits 0 und 1 müssen gleich sein 
#            ||     weil das Bit 0 übersprungen wird !
#pattern = 0b1110011_00110011_10101010_00101010
pattern = 0x0000_FFFF
pattern = 0x00FF_00FF
pattern = 0x0F0F_0F0F
pattern = 0x3333_3333  

sm = StateMachine(0, wave, out_base = Pin(16), freq=2048)
sm.put(pattern)        # pattern ins X-Register kopieren
sm.exec("pull()")
sm.exec("mov(x, osr)")

sm.active(1)
time.sleep(1.5)
sm.active(0)
    
print("\ndone")
